// Reference: https://www.d3-graph-gallery.com/graph/correlogram_scatter.html

// dimension of the whole chart. only one size since it has to be square
const marginWhole = { top: 75, right: 10, bottom: 10, left: 10 },
  sizeWhole = 750 - marginWhole.left - marginWhole.right;

// create the svg area
const sm_svg = d3
  .select("#scatterplot_matrix")
  .append("svg")
  .attr("width", sizeWhole + marginWhole.left + marginWhole.right)
  .attr("height", sizeWhole + marginWhole.top + marginWhole.bottom)
  .append("g")
  .attr("transform", `translate(${marginWhole.left},${marginWhole.top})`);

d3.csv("scatterplot_matrix.csv").then((data) => {
  // what are the numeric variables in this dataset? how many do I have?
  const allVar = [
    "height",
    "length",
    "width",
    "horsepower",
    "city_mpg",
    "highway_mpg",
  ];
  const numVar = allVar.length;

  // now I can compute the size of a single chart
  const mar = 20;
  size = sizeWhole / numVar;

  // ----------------- //
  // title
  // ----------------- //

  // add chart title
  sm_svg
    .append("text")
    .attr("x", sizeWhole / 1.5)
    .attr("y", -50)
    .attr("text-anchor", "middle")
    .text("Scatterplot Matrix of Car Data by Make");
  
 

  // ----------------- //
  // scales
  // ----------------- //

  // create a scale: gives the position of each pair each variable
  const position = d3
    .scalePoint()
    .domain(allVar)
    .range([0, sizeWhole - size]);

  // color scale: give me a make, I return a color
  const makes = Array.from(data, (d) => d.make);
  const makeColors = makes.map((el, i, arr) =>
    d3.interpolateTurbo(i / arr.length)
  );
  const color = d3.scaleOrdinal().domain(makes).range(makeColors);

  // ------------------------------- //
  // add charts
  // ------------------------------- //

  for (i in allVar) {
    for (j in allVar) {
      // get current variable name
      const var1 = allVar[i];
      const var2 = allVar[j];

      // if var1 == var2 i'm on the diagonal, I skip that
      if (var1 === var2) {
        continue;
      }

      // add x scale of each graph
      xextent = d3.extent(data, (d) => {
        return +d[var1];
      });
      const x = d3
        .scaleLinear()
        .domain(xextent)
        .nice()
        .range([0, size - 2 * mar]);

      // add y scale of each graph
      yextent = d3.extent(data, (d) => {
        return +d[var2];
      });
      const y = d3
        .scaleLinear()
        .domain(yextent)
        .nice()
        .range([size - 2 * mar, 0]);

      // add a 'g' at the right position
      const tmp = sm_svg
        .append("g")
        .attr(
          "transform",
          `translate(${position(var1) + mar},${position(var2) + mar})`
        );

      // add x and y axis in tmp
      tmp
        .append("g")
        .attr("transform", `translate(0,${size - mar * 2})`)
        .call(d3.axisBottom(x).ticks(3));
      tmp.append("g").call(d3.axisLeft(y).ticks(3));
      



      // add circle
      tmp
        .selectAll("myCircles")
        .data(data)
        .join("circle")
        .attr("cx", (d) => {
          return x(+d[var1]);
        })
        .attr("cy", (d) => {
          return y(+d[var2]);
        })
        .attr("r", 3)
        .attr("fill", (d) => {
          return color(d.make);
        });
      
      
    }
  }


  // ------------------------------- //
  // add variable names = diagonal
  // ------------------------------- //

  for (i in allVar) {
    for (j in allVar) {
      // if var1 == var2 I'm on the diagonal, otherwise I skip
      if (i != j) {
        continue;
      }
      // add text
      const var1 = allVar[i];
      const var2 = allVar[j];
      sm_svg
        .append("g")
        .attr("transform", `translate(${position(var1)},${position(var2)})`)
        .append("text")
        .attr("x", size / 2)
        .attr("y", size / 2)
        .text(var1)
        .attr("text-anchor", "middle");
      
      
    }
  }
  
// I ran out of time to do the legends

});
